# Generated by Django 3.1.4 on 2021-03-10 20:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("organizations", "0001_squashed_0008_auto_20201005_1552"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="organization",
            name="default_role",
        ),
        migrations.RemoveField(
            model_name="organizationmember",
            name="role",
        ),
    ]
