# Generated by Django 3.1.4 on 2021-03-08 11:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("projects", "0004_auto_20210306_0506"),
        ("projects", "0003_auto_20210305_1008"),
    ]

    operations = []
