# Generated by Django 3.1.4 on 2021-03-08 11:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("tasks", "0002_auto_20210304_1423"),
        ("tasks", "0002_auto_20210305_2035"),
    ]

    operations = []
